#include "StdInc.h"
#include "RollEm.h"

#ifdef _DEBUG

#define TEXT_X			8	
#define TEXT_Y			20
#define TEXT_WIDTH		140
#define TEXT_HEIGHT		110
#define TEXT_SPACING	2

#define DATABASE_RECORD	ROLLS_RECORD

Int g_iDatabaseListPos, g_iDatabaseListRows, g_iDatabaseListCols;

void InitDatabaseList ();
void DrawDatabaseList ();
void MoveDatabaseList (Int iDirection);

UInt GetDatabaseSize ()
{
    VoidHand hRecord;
    Ptr pRecord;
	UInt uiSize = 0;

	if (g_hRollEmDB)
	{
		if (DmNumRecords(g_hRollEmDB) > DATABASE_RECORD)
		{
			hRecord = DmGetRecord(g_hRollEmDB, DATABASE_RECORD);
			uiSize = MemHandleSize(hRecord);
			DmReleaseRecord(g_hRollEmDB, DATABASE_RECORD, false);  
		}
	}

	return uiSize;
}

Boolean DatabaseProc (EventPtr event)
{
    FormPtr pForm;
    Int handled = 0;

    switch (event->eType) 
    {
        case frmOpenEvent:
        {
            pForm = FrmGetActiveForm();
            FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_OK));
            FrmDrawForm(pForm);
			InitDatabaseList();
			DrawDatabaseList();
            handled = 1;
            break;
        }
        case ctlSelectEvent:
		{
            switch (event->data.ctlEnter.controlID)
            {
                case IDC_OK:
                    FrmReturnToForm(0);
                    break;
            }
            handled = 1;
            break;
		}
		case ctlRepeatEvent:
		{
            switch (event->data.ctlEnter.controlID)
            {
                case IDC_PREV:
                    MoveDatabaseList(-1);
                    break;
                case IDC_NEXT:
                    MoveDatabaseList(1);
                    break;
            }
            break;
		}
		case keyDownEvent:
		{
            switch (event->data.keyDown.chr)
            {
                case pageUpChr:
                    MoveDatabaseList(-1);
                    break;
                case pageDownChr:
                    MoveDatabaseList(1);
                    break;
            }
            break;
		}
	}

    return handled;
}

void InitDatabaseList ()
{
	// Modify the lists to look better.
	SortList();
	ReverseList();

	g_iDatabaseListPos = 0;
	g_iDatabaseListRows = (TEXT_HEIGHT - (TEXT_SPACING*0)) / FntCharHeight();
	g_iDatabaseListCols = 1;

	MoveDatabaseList(0);
}

void DrawDatabaseList ()
{
    Int iCurrentX=TEXT_X + TEXT_SPACING, iCurrentY=TEXT_Y, i;
    Int iCharHeight, iListSize, iEnd;
	Long lValue;
    Char szText [50];
    RectangleType rtRect;
    VoidHand hRecord;
    Ptr pRecord;
	UInt uiSize = 0;

    iCharHeight = FntCharHeight();
	iListSize = GetDatabaseSize();

	// Figure out the ending point.
	iEnd = g_iDatabaseListPos + g_iDatabaseListRows;
	if (iEnd > iListSize)
		iEnd = iListSize;

    rtRect.topLeft.x = TEXT_X;
    rtRect.topLeft.y = TEXT_Y;
    rtRect.extent.x = TEXT_WIDTH;
    rtRect.extent.y = TEXT_HEIGHT;

	WinDrawRectangleFrame(rectangleFrame, &rtRect);

    rtRect.topLeft.x++;
    rtRect.topLeft.y++;
    rtRect.extent.x -= 2;
    rtRect.extent.y -= 2;

	WinEraseRectangle(&rtRect, 0);

	if (g_hRollEmDB)
	{
		if (DmNumRecords(g_hRollEmDB) > DATABASE_RECORD)
		{
			hRecord = DmGetRecord(g_hRollEmDB, DATABASE_RECORD);
			pRecord = MemHandleLock(hRecord);
			for (i=g_iDatabaseListPos; i<iEnd; i++)
			{
				lValue = (Long)pRecord [i];
				StrLToA(szText, lValue);
				WinDrawChars(szText, StrLen(szText), iCurrentX, iCurrentY);
				iCurrentY += iCharHeight;
			}
			MemHandleUnlock(hRecord); 
			DmReleaseRecord(g_hRollEmDB, DATABASE_RECORD, false);  
		}
	}
}

void MoveDatabaseList (Int iDirection)
{
	FormPtr pForm;
	Word wUpIndex, wDownIndex;
	Boolean bScrollUp, bScrollDown;
	Int iNewPos;

	if (iDirection != 0)
	{
		iNewPos = g_iDatabaseListPos + iDirection;

		if ((iNewPos + g_iDatabaseListRows) > GetDatabaseSize())
			iNewPos = GetDatabaseSize() - g_iDatabaseListRows;
		if (iNewPos < 0)
			iNewPos = 0;

		if (iNewPos != g_iDatabaseListPos)
		{
			g_iDatabaseListPos = iNewPos;
			DrawDatabaseList();
		}
	}

	bScrollUp = (g_iDatabaseListPos != 0);
	bScrollDown = ((g_iDatabaseListPos + g_iDatabaseListRows) < GetDatabaseSize());

    pForm = FrmGetActiveForm();
	wUpIndex = FrmGetObjectIndex(pForm, IDC_PREV);
	wDownIndex = FrmGetObjectIndex(pForm, IDC_NEXT);
	FrmUpdateScrollers(pForm, wUpIndex, wDownIndex, bScrollUp, bScrollDown); 
}

#endif
