#include "StdInc.h"
#include "About.h"
#include "Custom.h"
#include "Database.h"
#include "Prefs.h"
#include "RollEm.h"
#include "RollFunc.h"
#include "RollResults.h"

Int StartApplication(void);
void StopApplication(void);
Boolean OpenDatabase(void);
Boolean CreateDatabase(void);
Boolean CloseDatabase(void);
Boolean ClearDatabase(void);
Boolean InitializeDatabase(void);
void EventLoop(void);

Boolean MainProc(EventPtr event);

Long RollDie (DieRollPtr pDieRoll);
void RollDice ();
void NextPage ();
void PrevPage ();
void ShowPage ();
void InitializePage ();
void ReadPage (Word wPage);
void WritePage (Word wPage);

void ReadRolls ();
void WriteRolls ();

void CreateTitle (DieRollPtr drPtr, CharPtr charPtr, Word wLen);
Boolean CheckValue (Byte iCompareType, Long lCompareValue, Long lResult);
void PlayBeep (Byte iBeepType);

void ChangeCurrent (Word wId);
void CustomizeButton (Word wId);
void CheckCompare (DieRollPtr pDieRoll, Long lResult);

Boolean Initialize ();
Boolean Deinitialize ();

void AddPage ();
void DeletePage ();
void ClearPage ();

void GetButton (Word wId, DieRollPtr drPtr);
void SetButton (Word wId, DieRollPtr drPtr);

DmOpenRef g_hRollEmDB;
Char szPrefsDatabaseName[] = "Prefs-RlEm";
Boolean g_bDoCustom;
Char g_szResultBuffer [MAX_TEMPSTRING_LEN];
Char g_szCurrentBuffer [MAX_TEMPSTRING_LEN];
ControlPtr g_cpResultButton;
VoidHand g_hPage;
PrefsPtr g_pPrefs;
ResultsPtr g_pResults;
DieRollPtr g_pDieRoll;
Word g_wCustomButton;

Char *g_szCompare [] = 
{
	"=", ">", "<", ">=", "<=", "!="
};

UChar g_szSounds [118] =
{
	0x00, 0x04, 
	0x01, 0xb8, 0x00, 0x4b, 
	0x00, 0x00, 0x00, 0x00, 
	0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xb8, 0x00, 0x4b, 
	0x02, 0x92, 0x00, 0x4b, 
	0x00, 0x00, 0x00, 0x00, 
	0x53, 0x75, 0x63, 0x63, 0x65, 0x73, 0x73, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x02, 0x92, 0x00, 0x4b,
	0x01, 0xb8, 0x00, 0x4b,
	0x00, 0x00, 0x00, 0x00,
	0x46, 0x61, 0x69, 0x6c, 0x75, 0x72, 0x65, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x4a, 0x00, 0x7d, 
	0x01, 0x4a, 0x00, 0x7d, 
	0x01, 0x4a, 0x00, 0x7d, 
	0x01, 0x04, 0x01, 0xf4,
	0x00, 0x00, 0x00, 0x00,
	0x35, 0x74, 0x68, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
    Int error;

    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
        error = StartApplication();	// Application start code
        if (error) return error;

        EventLoop();	// Event loop

        StopApplication ();	// Application stop code
    }
    return 0;
}

Int StartApplication(void)
{
	Int error;

	error = Initialize();
	if (error) 
		return error;
	error = OpenDatabase();
	if (error) 
		return error;

	FrmGotoForm(IDF_MAIN);
}

void StopApplication(void)
{
	// Write the prefs and results.
	OverwriteRecord(g_hRollEmDB, PREFS_RECORD, g_pPrefs, sizeof(Prefs));
	OverwriteRecord(g_hRollEmDB, RESULTS_RECORD, g_pResults, sizeof(Results));
	WriteRolls();

    // Save the current page before we leave.
    WritePage(g_pResults->wCurrentPage);

	RollFuncDestructor();
    CloseDatabase();
    Deinitialize();
}

Boolean OpenDatabase(void)
{
    g_hRollEmDB = DmOpenDatabaseByTypeCreator(ROLLEM_DBTYPE, ROLLEM_APPID, dmModeReadWrite);
    if (g_hRollEmDB) 
    {
		Word wVersion = 0;

		// Check the version number.
		ReadRecord(g_hRollEmDB, PREFS_RECORD, &wVersion, sizeof(Word));

		if (wVersion < VERSION_NUMBER)
		{
			if (wVersion > 0)
			{
				if (FrmCustomAlert(IDA_CLEAR_DATABASE, NULL, NULL, NULL) == 1)
				{
					FrmCustomAlert(IDA_UPDATE_DATABASE, NULL, NULL, NULL);
					return 1;
				}
			}
			ClearDatabase();
			InitializeDatabase();
		}

		return 0;
    }

	return CreateDatabase();
}

Boolean CreateDatabase(void)
{
    if (DmCreateDatabase(0, szPrefsDatabaseName, ROLLEM_APPID, ROLLEM_DBTYPE, false)) 
        return 1;
    g_hRollEmDB = DmOpenDatabaseByTypeCreator(ROLLEM_DBTYPE, ROLLEM_APPID, dmModeReadWrite);

	return InitializeDatabase();
}

Boolean CloseDatabase()
{
    // Close the database
//	DmResetRecordStates(g_hRollEmDB); // This crashes the app.
    DmCloseDatabase(g_hRollEmDB);

    return 0;
}

Boolean ClearDatabase()
{
	Int i;

	for (i=0; i<=g_pPrefs->wTotalPages; i++)
    {
		DmRemoveRecord(g_hRollEmDB, 0);
    }

    return 0;
}

Boolean InitializeDatabase(void)
{
	Word wTemp;
	UInt uiIndex;
	ULong ulLength;
	DieRollPtr pDieRoll;
	Int i;

	g_pPrefs->wVersion = VERSION_NUMBER;
    g_pPrefs->wTotalPages = DEFAULT_PAGES;
    g_pPrefs->wSaveRolls = 0;

	wTemp = 0;
	uiIndex = PREFS_RECORD;
	InsertRecord(g_hRollEmDB, uiIndex++, g_pPrefs, sizeof(Prefs));			// Prefs
	InsertRecord(g_hRollEmDB, uiIndex++, g_pResults, sizeof(Results));		// Results
	InsertRecord(g_hRollEmDB, uiIndex++, g_szSounds, sizeof(g_szSounds));	// Sounds
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));				// Rolls
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));				// Misc 1
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));				// Misc 2
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));				// Misc 3
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));				// Misc 4
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));				// Misc 5
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));				// Misc 6
    
    InitializePage();

	ulLength = sizeof(DieRoll) * NUM_BUTTONS;
    pDieRoll = (DieRollPtr)MemHandleLock(g_hPage);
    for (i=0; i<g_pPrefs->wTotalPages; i++)
    {
		InsertRecord(g_hRollEmDB, PAGE2REC(i), pDieRoll, ulLength);
    }
    MemHandleUnlock(g_hPage);

    return 0;
}

void EventLoop(void)
{
    short err;
    FormPtr pForm;
    EventType event;

    do
    {
        EvtGetEvent(&event, -1);

        if (SysHandleEvent(&event)) continue;
        if (MenuHandleEvent((void *)0, &event, &err)) continue;

        if (event.eType == frmLoadEvent)
        {
            pForm = FrmInitForm(event.data.frmLoad.formID);
            FrmSetActiveForm(pForm);
            switch (event.data.frmLoad.formID) 
            {
                case IDF_ABOUT:
                    FrmSetEventHandler(pForm, (FormEventHandlerPtr) AboutProc);
                    break;
                case IDF_CUSTOM:
                    FrmSetEventHandler(pForm, (FormEventHandlerPtr) CustomProc);
                    break;
                case IDF_MAIN:
                    FrmSetEventHandler(pForm, (FormEventHandlerPtr) MainProc);
                    break;
                case IDF_PREFS:
                    FrmSetEventHandler(pForm, (FormEventHandlerPtr) PrefsProc);
                    break;
#ifdef _DEBUG
                case IDF_ROLLS:
                    FrmSetEventHandler(pForm, (FormEventHandlerPtr) DatabaseProc);
                    break;
#else
				case IDF_ROLLS:
					FrmSetEventHandler(pForm, (FormEventHandlerPtr) RollsProc);
					break;
#endif
            }
        }
        FrmDispatchEvent(&event);
    } while(event.eType != appStopEvent);
}

Boolean MainProc(EventPtr event)
{
    Int handled = 0;

    switch (event->eType) 
    {
        case frmOpenEvent:
        {
            FormPtr pForm;

			RollFuncConstructor();

			// Read the prefs and results.
			ReadRecord(g_hRollEmDB, PREFS_RECORD, g_pPrefs, sizeof(Prefs));
			ReadRecord(g_hRollEmDB, RESULTS_RECORD, g_pResults, sizeof(Results));
			ReadRolls();

            pForm = FrmGetActiveForm();
            g_cpResultButton = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_RESULT));

            ReadPage(g_pResults->wCurrentPage);
            CtlSetLabel(g_cpResultButton, StrLToA(g_szResultBuffer, (Long)g_pResults->lLastResult));
            ChangeCurrent(g_pResults->wCurrentButton);

            FrmDrawForm(pForm);
            ShowPage();
            handled = 1;
            break;
        }
    
        case ctlSelectEvent:  // A control button was pressed and released.
            switch (event->data.ctlEnter.controlID)
            {
                case IDC_CUSTOM:
                    CustomizeButton(event->data.ctlEnter.controlID);
                    break;
                case IDC_BUTTON_0:
                case IDC_BUTTON_1:
                case IDC_BUTTON_2:
                case IDC_BUTTON_3:
                case IDC_BUTTON_4:
                case IDC_BUTTON_5:
                case IDC_BUTTON_6:
                case IDC_BUTTON_7:
                    if (g_bDoCustom)
                    {
						CustomizeButton(event->data.ctlEnter.controlID);
						break;
                    }
                    else
                    {
						ChangeCurrent(event->data.ctlEnter.controlID);
                    }
                case IDC_RESULT:
					RollDice();
					break;
                case IDC_VIEW_ROLLS:
                    FrmPopupForm(IDF_ROLLS);
                    break;
            }
            handled = 1;
            break;

		case ctlRepeatEvent:
            switch (event->data.ctlEnter.controlID)
            {
                case IDC_NEXT:
                    NextPage();
                    break;
                case IDC_PREV:
                    PrevPage();
                    break;
            }
            break;

        case menuEvent:
            switch (event->data.menu.itemID)
            {
                case IDM_OPTIONS_ABOUT:
                    FrmPopupForm(IDF_ABOUT);
                    break;
                case IDM_OPTIONS_PREFS:
                    FrmPopupForm(IDF_PREFS);
                    break;
                case IDM_RECORD_ADD:
                    AddPage();
                    break;
                case IDM_RECORD_DELETE:
                    DeletePage();
                    break;
                case IDM_RECORD_CLEAR:
                    ClearPage();
                    break;
            }
            handled = 1;
            break;
		case keyDownEvent:
            switch (event->data.keyDown.chr)
            {
                case pageUpChr:
                    PrevPage();
                    break;
                case pageDownChr:
                    NextPage();
                    break;
            }
            break;
    }
    return handled;
}

Long RollDie (DieRollPtr pDieRoll)
{
    Long lResult = pDieRoll->FIELD_MODIFIER;

	CreateList2(pDieRoll->FIELD_NUMBER, pDieRoll->FIELD_DIE);
    if ((pDieRoll->FIELD_BEST > 0) && (pDieRoll->FIELD_BEST < pDieRoll->FIELD_NUMBER))
	{
		SortList();
		SizeList(pDieRoll->FIELD_BEST);
	}

	if (pDieRoll->FIELD_SUCCESSVALUE > 0)
	{
		Int iSize, i; 

		lResult = 0;
		iSize = GetCurrentSizeList();
		for (i=0; i<iSize; i++)
		{
			if (CheckValue(pDieRoll->FIELD_SUCCESSCOMPARE, pDieRoll->FIELD_SUCCESSVALUE, GetElementList(i)))
				lResult++;
		}
	}
	else
	{
		lResult += SumList();
	}

    return lResult;
}

void CheckCompare (DieRollPtr pDieRoll, Long lResult)
{
    if (pDieRoll->FIELD_BEEPTYPE1)
    {
        if (CheckValue(pDieRoll->FIELD_BEEPCOMPARE1, pDieRoll->FIELD_BEEPVALUE1, lResult))
        {
			PlayBeep(pDieRoll->FIELD_BEEPTYPE1);
        }
    }
    if (pDieRoll->FIELD_BEEPTYPE2)
    {
        if (CheckValue(pDieRoll->FIELD_BEEPCOMPARE2, pDieRoll->FIELD_BEEPVALUE2, lResult))
        {
			PlayBeep(pDieRoll->FIELD_BEEPTYPE2);
        }
    }
}

Boolean CheckValue (Byte iCompareType, Long lCompareValue, Long lResult)
{
    switch(iCompareType)
    {
        case ComparisonEqual:
            return (lResult == lCompareValue);
        case ComparisonGreater:
            return (lResult > lCompareValue);
        case ComparisonLess:
            return (lResult < lCompareValue);
        case ComparisonGreaterEqual:
            return (lResult >= lCompareValue);
        case ComparisonLessEqual:
            return (lResult <= lCompareValue);
        case ComparisonNotEqual:
            return (lResult != lCompareValue);
        default:
            return false;
    }

}

void PlayBeep (Byte iBeepType)
{
	VoidHand hRecord;
	CharPtr pChar;
	SndCommandType sctSound;
	Int iCount;
 
	// Just making sure.
	if (iBeepType < 1)
		return;

	if (g_hRollEmDB)
	{
		if (DmNumRecords(g_hRollEmDB) > SOUNDS_RECORD)
		{
			hRecord = DmGetRecord(g_hRollEmDB, SOUNDS_RECORD);
			pChar = (CharPtr)MemHandleLock(hRecord);
			iCount = *((UIntPtr)pChar);

		    iBeepType--; // Position 0 is No beep
			if ((iCount > 0) && (iBeepType < iCount))
			{
				// Moveit 
				pChar += sizeof(UInt);
				while (iBeepType--)
				{
					// Skip the sounds.
					while (((NotePtr)pChar)->uiDuration)
						pChar += sizeof(Note);
					pChar += sizeof(Note);

					// Skip the titles.
					pChar += MAX_TITLE_LEN + 1;
				}

				// Setup the default sound.
				sctSound.cmd = sndCmdFreqDurationAmp;
				sctSound.param3 = sndDefaultAmp;

				// Play the sound.
				while (((NotePtr)pChar)->uiDuration)
				{
					if (((NotePtr)pChar)->uiHertz == 0)
					{
						Long lTemp;

						lTemp = ((Long)((NotePtr)pChar)->uiDuration * (Long)SysTicksPerSecond()) / 1000;
						if (lTemp)
							SysTaskDelay(lTemp);
					}
					else
					{
						sctSound.param1 = ((NotePtr)pChar)->uiHertz;
						sctSound.param2 = ((NotePtr)pChar)->uiDuration;
						SndDoCmd((VoidPtr)0, &sctSound, false);
					}
					pChar += sizeof(Note);
				}
			}
			MemHandleUnlock(hRecord); 
			DmReleaseRecord(g_hRollEmDB, SOUNDS_RECORD, false);  
		}
	}
}

void RollDice ()
{
	g_pResults->lLastResult = RollDie(g_pDieRoll);
	CtlSetLabel(g_cpResultButton, StrLToA(g_szResultBuffer, (Long)g_pResults->lLastResult));
	CheckCompare(g_pDieRoll, g_pResults->lLastResult);
}

void NextPage ()
{
    WritePage(g_pResults->wCurrentPage);
    g_pResults->wCurrentPage = (g_pResults->wCurrentPage + 1) % g_pPrefs->wTotalPages;
    ReadPage(g_pResults->wCurrentPage);
    ShowPage();
}

void PrevPage ()
{
    WritePage(g_pResults->wCurrentPage);
    g_pResults->wCurrentPage = (!g_pResults->wCurrentPage) ? g_pPrefs->wTotalPages - 1 : g_pResults->wCurrentPage - 1; 
    ReadPage(g_pResults->wCurrentPage);
    ShowPage();
}

void ShowPage ()
{
    FormPtr pForm;
    DieRollPtr pDieRoll;
    Char szPageNumber [3];
    RectangleType rtRect;
    Int i;
    FontID fiCurrent;

    pForm = FrmGetActiveForm();
    pDieRoll = (DieRollPtr) MemHandleLock(g_hPage);
    for (i=IDC_BUTTON_FIRST; i<=IDC_BUTTON_LAST; i++)
    {
        CtlSetLabel(FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, i)), 
            pDieRoll->szTitle);
        pDieRoll++;
    }
    MemHandleUnlock(g_hPage);

    fiCurrent = FntSetFont(1);

    rtRect.topLeft.x = 55;
    rtRect.topLeft.y = 141;
    rtRect.extent.x = 20;
    rtRect.extent.y = 13;

    WinEraseRectangle(&rtRect, 0);
    StrLToA(szPageNumber, (Long)(g_pResults->wCurrentPage + 1));
    DrawCenteredChars(szPageNumber, StrLen(szPageNumber), &rtRect); 
    //WinDrawRectangleFrame(rectangleFrame, &rtRect);

    FntSetFont(fiCurrent);
}

void InitializePage ()
{
    Int iDefaultButtons [NUM_BUTTONS] = { 2, 4, 6, 8, 10, 12, 20, 100 };
    Int i;
    DieRollPtr pDieRoll;

    pDieRoll = (DieRollPtr) MemHandleLock(g_hPage);
    MemSet(pDieRoll, (sizeof(DieRoll) * NUM_BUTTONS), '\0');
    for(i=IDC_BUTTON_FIRST; i<=IDC_BUTTON_LAST; i++)
    {
        pDieRoll->FIELD_DIE = iDefaultButtons [ID2NUM(i)];
        pDieRoll->FIELD_NUMBER = 1;
        pDieRoll->FIELD_MODIFIER = 0;

        CreateTitle(pDieRoll, pDieRoll->szTitle, MAX_TITLE_LEN+1);

        pDieRoll++;
    }
    MemHandleUnlock(g_hPage);
}

void ReadPage (Word wPage)
{
    VoidHand RecHandle;
    Ptr RecPointer;
    UInt uiIndex;
    ULong ulLength;
    DieRollPtr pDieRoll;

    wPage = PAGE2REC(wPage);

    ulLength = sizeof(DieRoll) * NUM_BUTTONS;
    RecHandle = DmGetRecord(g_hRollEmDB, wPage);
    RecPointer = MemHandleLock(RecHandle);
    pDieRoll = (DieRollPtr)MemHandleLock(g_hPage);
    MemMove(pDieRoll, RecPointer, ulLength);
    MemHandleUnlock(g_hPage);
    MemHandleUnlock(RecHandle); 
    DmReleaseRecord(g_hRollEmDB, wPage, false);  
}

void WritePage (Word wPage)
{
    VoidHand RecHandle;
    Ptr RecPointer;
    UInt uiIndex;
    ULong ulLength;
    DieRollPtr pDieRoll;

    wPage = PAGE2REC(wPage);

    ulLength = sizeof(DieRoll) * NUM_BUTTONS;
    RecHandle = DmGetRecord(g_hRollEmDB, wPage);
    RecPointer = MemHandleLock(RecHandle);
    pDieRoll = (DieRollPtr)MemHandleLock(g_hPage);
    DmWrite(RecPointer, 0, pDieRoll, ulLength);
    MemHandleUnlock(g_hPage);
    MemHandleUnlock(RecHandle); 
    DmReleaseRecord(g_hRollEmDB, wPage++, true);  
}

void CreateTitle (DieRollPtr pDieRoll, CharPtr charPtr, Word wLen)
{
	Char szTemp[MAX_TEMPSTRING_LEN];

	*szTemp = '\0';
    if ((pDieRoll->FIELD_BEST) && (pDieRoll->FIELD_BEST < pDieRoll->FIELD_NUMBER))
    {
        StrCat(szTemp, "Best ");
        StrLToA(&szTemp[StrLen(szTemp)], (Long)pDieRoll->FIELD_BEST);
        StrCat(szTemp, " of ");
        StrLToA(&szTemp[StrLen(szTemp)], (Long)pDieRoll->FIELD_NUMBER);
    }
    else
    {
        StrLToA(&szTemp[StrLen(szTemp)], (Long)pDieRoll->FIELD_NUMBER);
    }
    StrCat(szTemp, "d");
    StrLToA(&szTemp[StrLen(szTemp)], (Long)pDieRoll->FIELD_DIE);
    if ((pDieRoll->FIELD_MODIFIER != 0) && (pDieRoll->FIELD_SUCCESSVALUE == 0))
    {
        if (pDieRoll->FIELD_MODIFIER > 0)
            StrCat(szTemp, "+");
        StrLToA(&szTemp[StrLen(szTemp)], (Long)pDieRoll->FIELD_MODIFIER);
    }
	if (pDieRoll->FIELD_SUCCESSVALUE > 0)
	{
		StrCat(szTemp, " ");
		StrCat(szTemp, g_szCompare[pDieRoll->FIELD_SUCCESSCOMPARE]);
		StrCat(szTemp, " ");
		StrLToA(&szTemp[StrLen(szTemp)], (Long)pDieRoll->FIELD_SUCCESSVALUE);
	}

	if (wLen > MAX_TEMPSTRING_LEN)
		wLen = MAX_TEMPSTRING_LEN;
	StrNCopy(charPtr, szTemp, --wLen);
	charPtr[wLen] = '\0';
}

void ChangeCurrent (Word wId)
{
    ControlPtr pControl;
    FormPtr pForm;
 
    if ((wId < IDC_BUTTON_FIRST) || 
        (wId > IDC_BUTTON_LAST))
        wId = IDC_BUTTON_FIRST;

    g_pResults->wCurrentButton = wId;
    GetButton(wId, g_pDieRoll);

	CreateTitle(g_pDieRoll, g_szCurrentBuffer, MAX_TEMPSTRING_LEN);

    pForm = FrmGetActiveForm();
    pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_CURRENT));
    CtlSetLabel(pControl, g_szCurrentBuffer);
}

void GetButton (Word wId, DieRollPtr drPtr)
{
    DieRollPtr pDieRoll;

    pDieRoll = (DieRollPtr) MemHandleLock(g_hPage);
    MemMove(drPtr, &pDieRoll [ID2NUM(wId)], sizeof(DieRoll));
    MemHandleUnlock(g_hPage);
}

void SetButton (Word wId, DieRollPtr drPtr)
{
    DieRollPtr pDieRoll;

    pDieRoll = (DieRollPtr) MemHandleLock(g_hPage);
    MemMove(&pDieRoll [ID2NUM(wId)], drPtr, sizeof(DieRoll));
    MemHandleUnlock(g_hPage);
}

void GetCustomButton (DieRollPtr drPtr)
{
	GetButton(g_wCustomButton, drPtr);
}

void SetCustomButton (DieRollPtr drPtr)
{
	SetButton(g_wCustomButton, drPtr);
}

void CustomizeButton (Word wId)
{
    ControlPtr pControl;
    FormPtr pForm;

	pForm = FrmGetActiveForm();
	pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_CUSTOM));

    if (wId == IDC_CUSTOM)
    {
        g_bDoCustom = !g_bDoCustom;            
    }
    else if ((wId != IDC_CUSTOM) && (g_bDoCustom))
    {
		g_wCustomButton = wId;
		FrmPopupForm(IDF_CUSTOM);
		g_bDoCustom = false;
    }

	CtlSetLabel(pControl, (g_bDoCustom) ? "Cancel" : "Custom...");
}

Boolean Initialize ()
{
    g_pPrefs = MemPtrNew(sizeof(Prefs));
	g_pResults = MemPtrNew(sizeof(Results));
	g_pDieRoll = MemPtrNew(sizeof(DieRoll));
    g_hPage = MemHandleNew(sizeof(DieRoll) * NUM_BUTTONS);

	MemSet(g_pPrefs, sizeof(Prefs), '\0');
    MemSet(g_pResults, sizeof(Results), '\0');
    MemSet(g_pDieRoll, sizeof(DieRoll), '\0');

    g_bDoCustom = false;

    return 0;
}

Boolean Deinitialize ()
{
    if (g_pPrefs)
        MemPtrFree(g_pPrefs);
    if (g_pResults)
        MemPtrFree(g_pResults);
    if (g_pDieRoll)
        MemPtrFree(g_pDieRoll);
    if (g_hPage)
        MemHandleFree(g_hPage);

    return 0;
}

void AddPage ()
{
    VoidHand RecHandle;
    Ptr RecPointer;
    UInt uiIndex;
    ULong ulLength;

    if (g_pPrefs->wTotalPages < MAX_PAGES)
    {
        g_pPrefs->wTotalPages++;

		// Add a new record in the database.
		uiIndex = PAGE2REC(g_pPrefs->wTotalPages);
        ulLength = sizeof(DieRoll) * NUM_BUTTONS;
        DmNewRecord(g_hRollEmDB, &uiIndex, ulLength);
        DmReleaseRecord(g_hRollEmDB, uiIndex, true);  

		// Save the current page.
        WritePage(g_pResults->wCurrentPage);
        g_pResults->wCurrentPage = g_pPrefs->wTotalPages - 1;

		// Write and show the new page.
        InitializePage();
        WritePage(g_pResults->wCurrentPage);
        ShowPage();
    }
}

void DeletePage ()
{
    VoidHand RecHandle;
    Ptr RecPointer;
    UInt uiIndex;
    ULong ulLength;

    if (g_pPrefs->wTotalPages > MIN_PAGES)
    {
        if (!FrmCustomAlert(IDA_PAGE_CONFIRMATION, "delete", NULL, NULL))
        {
            uiIndex = PAGE2REC(g_pResults->wCurrentPage);
            DmRemoveRecord(g_hRollEmDB, uiIndex);
            g_pPrefs->wTotalPages--;
            if (g_pResults->wCurrentPage >= g_pPrefs->wTotalPages)
                g_pResults->wCurrentPage = g_pPrefs->wTotalPages - 1;
            ReadPage(g_pResults->wCurrentPage);
            ShowPage();
        }
    }
}

void ClearPage ()
{
    VoidHand RecHandle;
    Ptr RecPointer;
    UInt uiIndex;
    ULong ulLength;

    if (g_pPrefs->wTotalPages < MAX_PAGES)
    {
        if (!FrmCustomAlert(IDA_PAGE_CONFIRMATION, "clear", NULL, NULL))
        {
            InitializePage();
            WritePage(g_pResults->wCurrentPage);
            ShowPage();
        }
    }
}

void ReadRolls ()
{
    VoidHand hRecord;
    Ptr pRecord;
	UInt uiLength;
	LongPtr pLong;

	// Don't read in if we are not saving rolls.
	if (g_pPrefs->wSaveRolls == 0)
		return;

	hRecord = DmGetRecord(g_hRollEmDB, ROLLS_RECORD);
	uiLength = MemHandleSize(hRecord) / sizeof(Long);

	// If we have at least one read it.
	if (uiLength > 0)
	{
		CreateList1(uiLength);
		pLong = GetList();
		pRecord = MemHandleLock(hRecord);
		MemMove(pLong, pRecord, (uiLength * sizeof(Long)));
		MemHandleUnlock(hRecord); 
		ReleaseList();
	}
	DmReleaseRecord(g_hRollEmDB, ROLLS_RECORD, false);  
}

void WriteRolls ()
{
	if ((g_pPrefs->wSaveRolls == 0) || (GetCurrentSizeList() < 1))
	{
		Word wTemp = 0;
		OverwriteRecord(g_hRollEmDB, ROLLS_RECORD, &wTemp, sizeof(Word)); // Rolls
	}
	else
	{
		// Save the results.
		OverwriteRecord(g_hRollEmDB, ROLLS_RECORD, GetList(), (GetCurrentSizeList() * sizeof (Long))); // Rolls
		ReleaseList();
	}
}
