#include "StdInc.h"
#include "Misc.h"
#include "RollFunc.h"

#define MAX_LIST		5

typedef struct _tagRollList
{
	VoidHand hHandle;
	Int iTotalSize;
	Int iCurrentSize;
} RollList;
typedef RollList *RollListPtr;

RollList g_vRollStack [MAX_LIST];
Int g_iStackIndex;

/***************************************************************\
 *                     Helper functions                        *
\***************************************************************/

Int SortListCompare (VoidPtr ptr1, VoidPtr ptr2, Long other, 
                 SortRecordInfoPtr sortRecordInfoPtr1, 
                 SortRecordInfoPtr sortRecordInfoPtr2, 
                 VoidHand appInfoH)
//Int SortListCompare (VoidPtr ptr1, VoidPtr ptr2, Long other)
{
    return (*((Long*)ptr2) - *((Long*)ptr1));
}

/***************************************************************\
 *                    Language functions                       *
\***************************************************************/

void RollFuncConstructor ()
{
	Int i;

	for (i=0; i<MAX_LIST; i++)
	{
		g_vRollStack [g_iStackIndex].hHandle = 0;
		g_vRollStack [g_iStackIndex].iTotalSize = 0;
		g_vRollStack [g_iStackIndex].iCurrentSize = 0;
	}
	g_iStackIndex = 0;
}

void RollFuncDestructor ()
{
	Int i;

	for (i=0; i<MAX_LIST; i++)
	{
		g_iStackIndex = i;
		DestroyList();
	}
	g_iStackIndex = 0;
}

/***************************************************************\
 *                      List functions                         *
\***************************************************************/

void PushList ()
{
	if (g_iStackIndex < (MAX_LIST - 1))
		g_iStackIndex++;
}

void PopList ()
{
	if (g_iStackIndex > 0)
		g_iStackIndex--;
}

void CreateList1 (Int iSize)
{
	CreateList3(iSize, 0, 0, 0);
}

void CreateList2 (Int iSize, Int iDie)
{
	CreateList3(iSize, 1, iDie, 0);
}

void CreateList3 (Int iSize, Int iNumber, Int iDie, Int iModifier)
{
	LongPtr pLong;

	// Limit checks
	if (iSize < 0)
		iSize = 1;
	if (iNumber < 0)
		iNumber = 0;
	if (iDie < 0)
		iDie = 0;

	// Kill the old list.
	DestroyList();

	// Get the list data and check for errors.
	g_vRollStack [g_iStackIndex].hHandle = MemHandleNew(iSize * sizeof(Long));
	if (!g_vRollStack [g_iStackIndex].hHandle)
		return;

	// Initialize the list
	pLong = (LongPtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
	if (!iNumber || !iSize)
	{
		// Just initialize the list to zero
		MemSet(pLong, (iSize * sizeof(Long)), '\0');
	}
	else
	{
		// Initialize the list to a set of die rolls.
		Int i, t;

		for (i=0; i<iSize; i++)
		{
			*pLong = 0;
			for (t=0; t<iNumber; t++)
            {
                *pLong += (Long)(SysRandom(0) % iDie) + iModifier + 1;
            }
			pLong++;
		}
	}
	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	g_vRollStack [g_iStackIndex].iTotalSize = iSize;
	g_vRollStack [g_iStackIndex].iCurrentSize = iSize;
}

void DestroyList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle)
		MemHandleFree(g_vRollStack [g_iStackIndex].hHandle);
	g_vRollStack [g_iStackIndex].hHandle = 0;
	g_vRollStack [g_iStackIndex].iTotalSize = 0;
	g_vRollStack [g_iStackIndex].iCurrentSize = 0;
}

Int GetTotalSizeList ()
{
	return g_vRollStack [g_iStackIndex].iTotalSize;
}

Int GetCurrentSizeList ()
{
	return g_vRollStack [g_iStackIndex].iCurrentSize;
}

Long GetElementList (Int iIndex)
{
	LongPtr pLong;
	Long lValue;

	if (!g_vRollStack [g_iStackIndex].hHandle || (iIndex >= g_vRollStack [g_iStackIndex].iCurrentSize))
		return 0;

	// Get the value of the element
	pLong = (LongPtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
	lValue = pLong [iIndex];
	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);

	return lValue;
}

void SetElementList (Int iIndex, Long lValue)
{
	LongPtr pLong;

	if (!g_vRollStack [g_iStackIndex].hHandle || (iIndex >= g_vRollStack [g_iStackIndex].iCurrentSize))
		return;

	// St the value of the element
	pLong = (LongPtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
	pLong [iIndex] = lValue;
	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
}

LongPtr GetList ()
{
	if (!g_vRollStack [g_iStackIndex].hHandle)
		return 0;

	return (LongPtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
}

void ReleaseList ()
{
	if (!g_vRollStack [g_iStackIndex].hHandle)
		return;

	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
}

Long SumList ()
{
	LongPtr pLong;
	Long lValue;
	Int i;

	lValue = 0;
	if (g_vRollStack [g_iStackIndex].hHandle)
	{
		pLong = (LongPtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
		for (i=0; i<g_vRollStack [g_iStackIndex].iCurrentSize; i++)
		{
			lValue += pLong [i];
		}
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}

	return lValue;
}

Long MeanList ()
{
	Long lValue;

	if (g_vRollStack [g_iStackIndex].iCurrentSize == 0)
		return 0;

	// Mean with rounding
	lValue = SumList();
	lValue *= 10;
	lValue = (lValue / g_vRollStack [g_iStackIndex].iCurrentSize) + 5;
	lValue /= 10;

	return lValue;
}

Long MedianList ()
{
	Long lValue;

	if (g_vRollStack [g_iStackIndex].iCurrentSize == 0)
		return 0;

	SortList();

	// Median with rounding
	if (g_vRollStack [g_iStackIndex].iCurrentSize % 2)
	{
		lValue = GetElementList(g_vRollStack [g_iStackIndex].iCurrentSize / 2);
	}
	else
	{
		lValue = GetElementList(g_vRollStack [g_iStackIndex].iCurrentSize / 2);
		lValue += GetElementList((g_vRollStack [g_iStackIndex].iCurrentSize / 2) - 1);
		lValue *= 10;
		lValue = (lValue / 2) + 5;
		lValue /= 10;
	}

	return lValue;
}

Long ModeList ()
{
	Long lValue;

	lValue = 0;
	if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 0))
	{
		Int iLargestRun, iCurrentRun, iNumberOfValues, i;
		Long lLastValue;
		LongPtr pLong;

		SortList();

		pLong = (LongPtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
		if (g_vRollStack [g_iStackIndex].iCurrentSize == 1)
		{
			lValue = *pLong;
		}
		else
		{
			iLargestRun = 1;
			iCurrentRun = 1;
			iNumberOfValues = 0;
			lLastValue = *pLong;
			for (i=1; i<g_vRollStack [g_iStackIndex].iCurrentSize; i++)
			{
				if (pLong [i] != lLastValue)
				{
					if (iCurrentRun == iLargestRun)
					{
						lValue += lLastValue;
						iNumberOfValues++;
					}
					else if (iCurrentRun > iLargestRun)
					{
						lValue = lLastValue;
						iNumberOfValues = 1;
						iLargestRun = iCurrentRun;
					}
					iCurrentRun = 0;
				}
				lLastValue = pLong [i];
				iCurrentRun++;
			}
			if (iCurrentRun == iLargestRun)
			{
				lValue += lLastValue;
				iNumberOfValues++;
			}
			else if (iCurrentRun > iLargestRun)
			{
				lValue = lLastValue;
				iNumberOfValues = 1;
				iLargestRun = iCurrentRun;
			}

			lValue *= 10;
			lValue = (lValue / iNumberOfValues) + 5;
			lValue /= 10;
		}
		
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}

	return lValue;
}

void SizeList (Int iSize)
{
	if ((iSize < 0) || (iSize >= g_vRollStack [g_iStackIndex].iTotalSize))
		return;
	g_vRollStack [g_iStackIndex].iCurrentSize = iSize;
}

void SortList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 0))
	{
		BytePtr pByte;

		pByte = (BytePtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
        SysQSort(pByte, g_vRollStack [g_iStackIndex].iCurrentSize, sizeof(Long), SortListCompare, 0);
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}
}

void ShuffleList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 0))
	{
		BytePtr pByte;

		pByte = (BytePtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
        QShuffle(pByte, g_vRollStack [g_iStackIndex].iCurrentSize, sizeof(Long));
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}
}

void ReverseList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 1))
	{
		LongPtr pLong;
		Long lValue;
		Int iStart, iEnd;

		iStart = 0;
		iEnd = g_vRollStack [g_iStackIndex].iCurrentSize - 1;

		pLong = (LongPtr)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
		while (iStart < iEnd)
		{
			lValue = pLong [iStart];
			pLong [iStart++] = pLong [iEnd];
			pLong [iEnd--] = lValue;
		}
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}
}
